/* ============================================================
 * File  : channelbrowser.cpp
 * Author: Alvaro Gonzalez Sola <cojonuo@gmail.com>
 * Date  : 2008-08-18
 * Description :
 *
 * ChannelBrowser interface
 *
 * Copyright 2008 by Alvaro Gonzalez Sola
 *
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published bythe Free Software Foundation;
 * either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * ============================================================ */

#ifndef CHANNELBROWSER_CPP
#define CHANNELBROWSER_CPP

// C Headers
#include <unistd.h>
#include <math.h>

// QT headers
#include <qapplication.h>
#include <q3network.h>
#include <qdir.h>

//Added by qt3to4:
#include <QUrl>

// MythTV headers
#include <mythtv/libmythui/mythmainwindow.h>
#include <mythtv/util.h>
#include <mythtv/httpcomms.h>
#include <mythtv/mythcontext.h>
#include <mythtv/mythdbcon.h>
#include <mythtv/mythdirs.h>

using namespace std;

#include "channelbrowser.h"
#include "searchscreen.h"
#include "channelscreen.h"
#include "tubedbutil.h"
#include "channeleditor.h"

#include <qfile.h>
#include <qstringlist.h>
#include <qdatastream.h>


//========================================================================================
//              Constructor, Desctructor and setup function
//========================================================================================


ChannelBrowser::ChannelBrowser(MythScreenStack *parent, const char *name) 
    : MythScreenType (parent, name)
{

    m_channelList = NULL;
    m_infoText = NULL;

    m_channels.setAutoDelete(true);
    

//     LoadChannels();

    //This only provisional, need load from database the current channel selected from config dialog
    //     m_channels.append(new FeedChannel("Mosaic","http://www.linktv.org/rss/hq/mosaic.xml"));
    //     m_channels.append(new FeedChannel("La Dosis Diaria","http://dosisdiaria.mobuzz.tv/rss/itunes.xml"));
    //     m_channels.append(new FeedChannel("Nasa","http://www.jpl.nasa.gov/multimedia/rss/podfeed-hd.xml"));
    //     m_channels.append(new FeedChannel("whakatikey","http://richie-b.blip.tv/posts/?skin=rss"));
    //     m_channels.append(new FeedChannel("wired","http://feedproxy.feedburner.com/pbs/kcet/wiredscience-video"));
    //     m_channels.append(new FeedChannel("google trailer","http://video.google.com/videofeed?type=search&q=type:movie_trailer&so=1"));
    // 
//         m_channels.append(new Channel("Search","",Channel::Search));a

}

ChannelBrowser::~ChannelBrowser(){}


bool ChannelBrowser::Create(){

    VERBOSE(VB_IMPORTANT, "CREATE");

    bool foundtheme = false;

    foundtheme = LoadWindowFromXML("tube-ui.xml", "channel", this);

    if (!foundtheme)
        return false;

    m_channelList = dynamic_cast<MythUIButtonList *> (GetChild("channellist"));
    m_infoText = dynamic_cast<MythUIText *> (GetChild("infotext"));

    if (!m_channelList || !m_infoText)
    {
        VERBOSE(VB_IMPORTANT, "Theme is missing critical theme elements.");
        return false;
    }

    if (!BuildFocusList())
        VERBOSE(VB_IMPORTANT, "Failed to build a focuslist. Something is wrong");

    connect(m_channelList, SIGNAL(itemClicked( MythUIButtonListItem*)),
            this, SLOT( selectChannelBrowser(MythUIButtonListItem*)));

    SetFocusWidget(m_channelList);

    LoadChannels();
    LoadChannels();

    return true;
}

bool ChannelBrowser::keyPressEvent(QKeyEvent *event){

    if (GetFocusWidget()->keyPressEvent(event))
        return true;

    bool handled = false;
    QStringList actions;
    gContext->GetMainWindow()->TranslateKeyPress("ChannelBrowser", event, actions);

    for (int i = 0; i < actions.size() && !handled; i++)
    {
        QString action = actions[i];
        handled = true;

        if (action == "MENU"){
                ShowMenu();
        }else{
            handled = false;
        }
    }

    if (!handled && MythScreenType::keyPressEvent(event))
        handled = true;

    return handled;
}

void ChannelBrowser::selectChannelBrowser(MythUIButtonListItem* selected){

    if (!selected)
        return;

    Channel * prog = NULL;

    prog = (Channel *) selected->getData();

    //diferent between search channel and other channel
    if(prog){
        if(prog->type() == Channel::Search){

            MythScreenStack *mainStack = GetMythMainWindow()->GetMainStack();

            SearchScreen *mythtube = new SearchScreen(mainStack, "mythtube");

            connect(mythtube, SIGNAL(Exiting()), this, SLOT(LoadChannels()));

            if (mythtube->Create())
                mainStack->AddScreen(mythtube);

        }else{

            MythScreenStack *mainStack = GetMythMainWindow()->GetMainStack();

            FeedChannel * feed_prog = (FeedChannel *) selected->getData();

            ChannelScreen *channelScreen = new ChannelScreen(mainStack, "channelselected", feed_prog);

            connect(channelScreen, SIGNAL(Exiting()), this, SLOT(LoadChannels()));

            if (channelScreen->Create())
                mainStack->AddScreen(channelScreen);
        }
    }

}

void ChannelBrowser::customEvent(QEvent *event)
{

    if (event->type() == kMythDialogBoxCompletionEventType)
    {
        DialogCompletionEvent *dce =
                                dynamic_cast<DialogCompletionEvent*>(event);

        QString resultid= dce->GetId();
        int buttonnum  = dce->GetResult();

        if (resultid == "options")
        {
            if (buttonnum == 0)
            {
                ShowEditDialog(false);
            }
            else if (buttonnum == 1)
            {
                ShowEditDialog(true);
            }
            else if (buttonnum == 2)
            {
                deleteChannel();
            }

        }

        m_menuPopup = NULL;
    }

}

void ChannelBrowser::ShowMenu(){
    QString label = tr("Options");

    MythScreenStack *popupStack =
                            GetMythMainWindow()->GetStack("popup stack");

    m_menuPopup = new MythDialogBox(label, popupStack, "mythtubemenupopup");

    if (m_menuPopup->Create())
        popupStack->AddScreen(m_menuPopup);

    m_menuPopup->SetReturnEvent(this, "options");

    m_menuPopup->AddButton(tr("Add Channel"));
    m_menuPopup->AddButton(tr("Edit Channel"));
    m_menuPopup->AddButton(tr("Delete Channel"));
    m_menuPopup->AddButton(tr("Cancel"));
}

void ChannelBrowser::ShowEditDialog(bool edit)
{
    Channel *channel = NULL;

    if (edit)
    {
        MythUIButtonListItem *channelListItem = m_channelList->GetItemCurrent();

        if (channelListItem && channelListItem->getData())
            channel = (Channel*) channelListItem->getData();
    }


    MythScreenStack *mainStack = GetMythMainWindow()->GetMainStack();

    ChannelsEditor *channeleditor = new ChannelsEditor(channel, edit, mainStack,"mythchaneleditor");

    connect(channeleditor, SIGNAL(Exiting()), this, SLOT(LoadChannels()));


    if (channeleditor->Create())
        mainStack->AddScreen(channeleditor);

}

void ChannelBrowser::deleteChannel()
{
    MythUIButtonListItem *channelUIItem = m_channelList->GetItemCurrent();

    QString channelName;
    if (channelUIItem && channelUIItem->getData())
    {
        Channel *channel = (Channel*) channelUIItem->getData();
        if(channel)
        {
            channelName = channel->name();

            removeFromDB(channelName);
            LoadChannels();
        }
    }
}

void ChannelBrowser::LoadChannels(){

    m_channels.clear();

    m_channels = loadChannels();

    m_channelList->Reset();

    m_channels.append(new Channel("Search","",Channel::Search));

    //add all channel on m_channels
    for (Channel *program = m_channels.first(); program; program = m_channels.next())
    {

        //no esta del todo bien esto por ejemplo no carga tras volver al browser despues de un retrieve
        //ademas es necesario un retrieve para obtener el logo (no se muy bien como arreglar eso)
        MythImage *image = GetMythPainter()->GetFormatImage();

        QString imgFile = program->channelDir() + "/logo";

        if(QFile::exists(imgFile)){
            image->Load(imgFile);
            MythUIButtonListItem* item =
                new MythUIButtonListItem(m_channelList, program->name(),image);
            item->setData(program);
        }
        else{
            MythUIButtonListItem* item =
                new MythUIButtonListItem(m_channelList, program->name());
            item->setData(program);
        }

    }

}

#endif
