/* ============================================================
 * File  : channelscreen.cpp
 * Author: Alvaro Gonzalez Sola <cojonuo@gmail.com>
 * Date  : 2008-08-18
 * Description :
 *
 * Channel interface
 *
 * Copyright 2008 by Alvaro Gonzalez Sola
 *
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published bythe Free Software Foundation;
 * either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * ============================================================ */

#ifndef CHANNELSCREEN_CPP
#define CHANNELSCREEN_CPP

// C Headers
#include <unistd.h>
#include <math.h>

// QT headers
#include <qapplication.h>
#include <q3network.h>
#include <qdir.h>

//Added by qt3to4:
#include <QUrl>

// MythTV headers
#include <mythtv/libmythui/mythmainwindow.h>
#include <mythtv/util.h>
#include <mythtv/httpcomms.h>
#include <mythtv/mythcontext.h>
#include <mythtv/mythdbcon.h>
#include <mythtv/mythdirs.h>

using namespace std;

#include "channelscreen.h"

#include <qfile.h>
#include <qstringlist.h>
#include <qdatastream.h>

// 

//========================================================================================
//              Constructor, Desctructor and setup function
//========================================================================================


ChannelScreen::ChannelScreen(MythScreenStack *parent, const char *name) 
    : MythTubeScreen (parent, name)
{

    m_videoList = NULL;
    m_nameText = NULL;
    m_channel = NULL;
}

ChannelScreen::ChannelScreen(MythScreenStack *parent, const char *name, FeedChannel * channel) 
    : MythTubeScreen (parent, name)
{

    m_videoList = NULL;
    m_nameText = NULL;
    m_descText = NULL;
    m_channel = channel;
}

ChannelScreen::~ChannelScreen(){}


bool ChannelScreen::Create(){

    VERBOSE(VB_IMPORTANT, "CREATE");

    bool foundtheme = false;

    foundtheme = LoadWindowFromXML("tube-ui.xml", "channelselected", this);

    if (!foundtheme)
        return false;

    m_videoList = dynamic_cast<MythUIButtonList *> (GetChild("videolist"));
    m_nameText = dynamic_cast<MythUIText *> (GetChild("nametext"));
    m_titleText = dynamic_cast<MythUIText *> (GetChild("title"));
    m_descText = dynamic_cast<MythUIText *> (GetChild("desctext"));
    m_channelThumbnail = dynamic_cast<MythUIImage *> (GetChild("channelthumbnail"));
    m_videoThumbnail = dynamic_cast<MythUIImage *> (GetChild("videothumbnail"));
    m_enclosureInfo = dynamic_cast<MythUIImage *> (GetChild("enclosureinfo"));

    if (!m_videoList || !m_nameText || !m_descText || !m_titleText || !m_channelThumbnail ||
        !m_videoThumbnail || !m_enclosureInfo)
    {
        VERBOSE(VB_IMPORTANT, "Theme is missing critical theme elements.");
        return false;
    }

    if (!BuildFocusList())
        VERBOSE(VB_IMPORTANT, "Failed to build a focuslist. Something is wrong");

    m_enclosureInfo->Hide();

    SetFocusWidget(m_videoList);

    m_nameText->SetText(m_channel->name());

    showInfo("retrieving");

    m_channel->retrieve();

    connect(m_channel, SIGNAL(finished(FeedChannel*)),
                this, SLOT(showResult(FeedChannel*)));
    connect(m_videoList, SIGNAL(itemClicked( MythUIButtonListItem*)),
            this, SLOT( viewVideo(MythUIButtonListItem*)));
    connect(m_videoList, SIGNAL(itemSelected( MythUIButtonListItem*)),
            this, SLOT( updateInfoView(MythUIButtonListItem*)));

    return true;
}

bool ChannelScreen::keyPressEvent(QKeyEvent *event){

    if (GetFocusWidget()->keyPressEvent(event))
        return true;

    bool handled = false;
    QStringList actions;
    gContext->GetMainWindow()->TranslateKeyPress("ChannelScreen", event, actions);

    for (int i = 0; i < actions.size() && !handled; i++)
    {
        QString action = actions[i];
        handled = true;

        if (action == "MENU"){
                ShowMenu();
        }else{
            handled = false;
        }
    }

    if (!handled && MythTubeScreen::keyPressEvent(event))
        handled = true;

    return handled;
}

void ChannelScreen::showResult(FeedChannel* channel){

    clearInfo();

    channel->process();

    m_videoList->Reset();

    QString url1 = channel->imageURL();

    QString sFilename1 = QString("%1/%2")
                    .arg(channel->channelDir()).arg("logo");


    //ojo este metodo solo descarga una vez el logo si luego cambia el logo del canal no lo vuelve a descargar
    bool exists1 = QFile::exists(sFilename1);
    if (!exists1)
        HttpComms::getHttpFile(sFilename1, url1, 20000);

    m_channelThumbnail->SetFilename(sFilename1);
    m_channelThumbnail->Load();

    for (TubeVideo* video = channel->videoList().first(); video;
            video = channel->videoList().next()) {

        MythImage *image = GetMythPainter()->GetFormatImage();

        QString fileprefix = configDir("/Tubecache");

        QString url = video->thumbnail();

        if(!url.isEmpty()){

            QString sFilename = QString("%1/%2")
                    .arg(fileprefix).arg(qChecksum(url,url.length()));

            bool exists = QFile::exists(sFilename);
            if (!exists)
                HttpComms::getHttpFile(sFilename, url, 20000);

    //         if (m_thumbnailImage)
    //         {
    //             m_thumbnailImage->SetFilename(sFilename);
    //             m_thumbnailImage->Load();
    // 
    //             if (!m_thumbnailImage->IsVisible())
    //                 m_thumbnailImage->Show();
    //         }

            image->Load(sFilename);
        }else{
            image->Load("~/.mythtv/MythTube/Tubecache/10105");
        }
//         else{
//             image->Load("/home/cojonuo/.mythtv/MythTube/Tubecache/10105");
//         }

        MythUIButtonListItem* item =
            new MythUIButtonListItem(m_videoList, video->title(),image);
        item->setData(video);
    }

    if(!m_videoList->IsEmpty())
        updateInfoView(m_videoList->GetItemCurrent());

}

void ChannelScreen::viewVideo(MythUIButtonListItem* selected){

    TubeVideo *video = (TubeVideo*) selected->getData();

    VERBOSE(VB_GENERAL, QString("reproduciendo Stream"));

    //si esta en cache lo uso de ahi de ahi
    if(onCache(video)){
        VERBOSE(VB_GENERAL, QString("video cacheado"));

        QString file = getCache(video);

        if(m_mode == SAVE){
            m_mode = DOWNLOAD;
            VERBOSE(VB_GENERAL, QString("guardando Video"));

            QString fileprefix = configDir("/saveVideos");

            QFile savefile(file);

            savefile.copy(fileprefix + "/" + video->title());

        }else{
            qApp->unlock();
            playVideo(file);
            qApp->lock();
        }

    }else if(m_mode == STREAM){  //si es un stream lo reproduzco
        m_mode = DOWNLOAD;
        VERBOSE(VB_GENERAL, QString("reproduciendo Stream"));

        qApp->unlock();
        playVideo("\""+video->enclosure()+"\"");
        qApp->lock();

    }else if(m_mode == SAVE){
        m_mode = DOWNLOAD;
        VERBOSE(VB_GENERAL, QString("guardando Video"));

        QString fileprefix = configDir("/saveVideos");

        downloadVideo(video, fileprefix);
    }else{
        downloadVideo(video);
    }

}

void ChannelScreen::updateInfoView(MythUIButtonListItem * selected){

    TubeVideo *video = (TubeVideo*) selected->getData();

    if(video){
        m_descText->SetText(video->description());
        m_titleText->SetText(video->title());

        if((video->enclosure()).isEmpty()){
            m_enclosureInfo->Show();
        }else{
            m_enclosureInfo->Hide();
        }

        QString url1 = video->thumbnail();

        QString sFilename1 = QString("%1/%2")
                        .arg(configDir("/Tubecache")).arg(qChecksum(url1,url1.length()));

        bool exists1 = QFile::exists(sFilename1);
        if (!exists1)
            HttpComms::getHttpFile(sFilename1, url1, 20000);

        m_videoThumbnail->SetFilename(sFilename1);
        m_videoThumbnail->Load();

    }
}

//reproduce un video apartir de una direccion
//OLD CODE
/*
void ChannelScreen::playVideo(const QString &filename)
{
    QString command_string = gContext->GetSetting("VideoDefaultPlayer");

    gContext->sendPlaybackStart();

    if ((command_string.find("Internal", 0, false) > -1) ||
        (command_string.length() < 1))
    {
        command_string = "Internal";
        gContext->GetMainWindow()->HandleMedia(command_string, filename);
    }
    else
    {
        if (command_string.contains("%s"))
            command_string = command_string.replace(QRegExp("%s"), filename);

        myth_system(command_string);
    }

    gContext->sendPlaybackEnd();
}
*/

void ChannelScreen::customEvent(QEvent *event)
{

    if (event->type() == kMythDialogBoxCompletionEventType)
    {
        DialogCompletionEvent *dce =
                                dynamic_cast<DialogCompletionEvent*>(event);

        QString resultid= dce->GetId();
        int buttonnum  = dce->GetResult();

        if (resultid == "options")
        {
            if (buttonnum == 0)
            {
                StreamVideo();
            }
            else if (buttonnum == 1)
            {
                SaveVideo();
            }
        }

        m_menuPopup = NULL;
    }

}

void ChannelScreen::ShowMenu(){
    QString label = tr("Options");

    MythScreenStack *popupStack =
                            GetMythMainWindow()->GetStack("popup stack");

    m_menuPopup = new MythDialogBox(label, popupStack, "mythtubemenupopup");

    if (m_menuPopup->Create())
        popupStack->AddScreen(m_menuPopup);

    m_menuPopup->SetReturnEvent(this, "options");

    m_menuPopup->AddButton(tr("View Stream"));
    m_menuPopup->AddButton(tr("Save"));
    m_menuPopup->AddButton(tr("Cancel"));
}

void ChannelScreen::StreamVideo(){
    m_mode = STREAM;
    viewVideo(m_videoList->GetItemCurrent());
}

void ChannelScreen::SaveVideo(){
    m_mode = SAVE;
    viewVideo(m_videoList->GetItemCurrent());
}

#endif
