/* ============================================================
 * File  : mythtubescreen.h
 * Author: Alvaro Gonzalez Sola <cojonuo@gmail.com>
 * Date  : 2008-08-18
 * Description :
 *
 * Generic mythtube interface
 *
 * Copyright 2008 by Alvaro Gonzalez Sola
 *
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published bythe Free Software Foundation;
 * either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * ============================================================ */

#ifndef MYTHTUBESCREEN_H
#define MYTHTUBESCREEN_H

#include <mythtv/libmythui/mythscreentype.h>
#include <mythtv/libmythui/mythuitext.h>
#include <mythtv/libmythui/mythuitextedit.h>
#include <mythtv/libmythui/mythlistbutton.h>
#include <mythtv/libmythui/mythuibuttonlist.h>
#include <mythtv/libmythui/mythuiimage.h>
#include <mythtv/libmythui/mythdialogbox.h>
#include <mythtv/libmythui/mythprogressdialog.h>
#include <mythtv/httpcomms.h>

#include <qstring.h>
#include <QHash>

#include "tubevideo.h"

class MythTubeScreen : public MythScreenType
{
public:

    enum VideoMode {
        DOWNLOAD = 0,
        STREAM,
        SAVE
    };

    typedef QHash<QString, QString> VideoCache;

    struct Cache{
        int size;
        int index;
        VideoCache content;
    };

    MythTubeScreen (MythScreenStack* parent, const char* name );
    ~MythTubeScreen ( );
    virtual bool Create ( ) = 0;
    virtual bool keyPressEvent (QKeyEvent* event );
    bool downloadVideo (TubeVideo* video, QString path = NULL);
    bool getHttpFile (QString sFilename, QString cmdURL );
    void createProgress (QString title );
    void showInfo (QString message );
    void clearInfo ();
    QString configDir (QString path = NULL);
    void playVideo (QString filename );

    bool onCache(TubeVideo *video);
    QString getCache(TubeVideo *video);

private:

    //Info elements
    MythUIProgressDialog* m_progressPopup;
    MythDialogBox* m_infoPopup;

    //Video cache
    Cache m_cache;

    //Download element
    HttpComms* m_httpGrabber;
    bool m_abortHttp;
//     VideoMode m_m_mode;
//     ChannelEngine* m_engine;

protected:

    VideoMode m_mode;

};

#endif // CHANNELSCREEN_H
