#!/usr/bin/perl -w

#
# This script search and get video data from alpacine
# and return a xml archive
#
# Author: Alvaro Gonzalez Sola (cojonuo@gmail.com)
# v0.8
#
# changes:
#
#   v0.8 (18/08/2008) (Alvaro Gonzalez Sola):
#         - initial version
#

use LWP::UserAgent;
use URI::Escape;
use HTML::Entities;

use Encode;

eval "use DateTime::Format::Strptime"; my $has_date_format = $@ ? 0 : 1;

use vars qw($opt_h $opt_r $opt_d $opt_i $opt_v $opt_S $opt_M);
use Getopt::Std; 

$title = "Alpacine search"; 
$version = "v0.8";
$author = "Alvaro Gonzalez Sola (cojonuo\@gmail.com)";

binmode(STDOUT, ":utf8");

# display usage
sub usage {
   print "usage: $0 -hdrviMPD [parameters]\n";
   print "       -h           help\n";
   print "       -d           debug\n";
   print "       -r           dump raw query result data only\n";
   print "       -v           display version\n";
   print "       -i           display info\n";
   print "\n";
   print "       -M <query>    search videos\n";
   exit(-1);
}

# display 1-line of info that describes the version of the program
sub version {
   print "$title ($version) by $author\n"
} 

# display 1-line of info that can describe the type of query used
sub info {
   print "Search videos on alpacine.\n";
}

# display detailed help 
sub help {
   version();
   info();
   usage();
}

# returns text within 'data' between 'beg' and 'end' matching strings
sub parseBetween {
   my ($data, $beg, $end)=@_; # grab parameters

   my $ldata = lc($data);
   my $start = index($ldata, lc($beg)) + length($beg);
   my $finish = index($ldata, lc($end), $start);
   if ($start != (length($beg) -1) && $finish != -1) {
      my $result = substr($data, $start, $finish - $start);
      decode_entities($result);
      return $result;
   }
   return "";
}

# search and dump result on xml format
sub search {
  my ($filename)=@_; # grab parameters # adquiere los parametros

   # If we wanted to inspect the file for any reason we can do that now

   #
   # Convert filename into a query string 
   # (use same rules that Metadata::guesTitle does)

   my $query = $filename;
   $query = uri_unescape($query);  # in case it was escaped

   # Strip off the file extension
   if (rindex($query, '.') != -1) {
      $query = substr($query, 0, rindex($query, '.'));
   }

   # Strip off anything following '(' - people use this for general comments
   if (rindex($query, '(') != -1) {
      $query = substr($query, 0, rindex($query, '(')); 
   }

   # Strip off anything following '[' - people use this for general comments
   if (rindex($query, '[') != -1) {
      $query = substr($query, 0, rindex($query, '[')); 
   }

   # prepare the url
   $query = uri_escape($query);
   if (defined $opt_d) { 
      printf("# query: '%s'\n", $query);
   }

   # get the search results  page
   my $request="http://alpacine.com/buscar/?buscar=$query";

   my $ua = LWP::UserAgent->new;
   $ua->timeout(100);
   $ua->env_proxy;
   my $response = $ua->get($request);
   if (!$response->is_success){die $response->status_line;}{
        if (defined $opt_h){print $response->content;}
   }

   # parse movie list from matches
   my $beg = "<li>";
   my $end = "</li>";
   my $count = 0;
   my @movies;

   my $data = parseBetween($response->content,"<span class=\"resultados\">","</ul>");
   if ($data eq "") {
      printf("# no results\n");
      return;
   }

   my $start = index($data, $beg);
   my $finish = index($data, $end, $start);
   my $year;
   my $title;

   print "<?xml version='1.0' encoding='UTF-8'?><feed>";
   #Extract movie list, with data
   while ($start != -1 && $start < length($data)) {
      $start += length($beg);
      my $entry = substr($data, $start, $finish - $start);
      $start = index($data, $beg, $finish + 1);
      $finish = index($data, $end, $start);

      my $title = "";
      my $year = "";
      my $movienum = "";

      if($entry =~ /.*\/pelicula\/(\d+)\/">(.*)<\/a> \((\d+)\)/){
          $movienum = $1;
           $title = $2;
           $year = $3;
      } else {
           if (defined $opt_d) {
               print("Unrecognized entry format ($entry)\n");
           }
           next;
      }

      my $skip = 0;

      # check if movie have trailer
      my $request2 = "http://www.alpacine.com/pelicula/" . $movienum . "/";
      if (defined $opt_d) { printf("# request: '%s'\n", $request2); }

      my $ua2 = LWP::UserAgent->new;
      $ua2->timeout(10);
      $ua2->env_proxy;
      my $response2 = $ua2->get($request2);
      if (!$response2->is_success){die $response2->status_line;}
      if (defined $opt_r) { print $response2->content; }

      my $data2 = parseBetween($response2->content,"<b>Trailer</b>","</div></div>");
      my $data3 = parseBetween($response2->content,"<div class=\"imagen\">","/>");
      my $thumbnail = parseBetween($data3,"src=\"","\"");

      #if hava trailer
      if($data2){

        $data2 =~ m/.*<a href="(.*)".*src="(.*)" \/>/;
        my $url = $1;
        $thumbnail_aux = $2;

        if(!$thumbnail){$thumbnail = $thumbnail_aux;}

        $url = "http://alpacine.com$url";

        my $content = parseBetween($response2->content,"Sinopsis:</div><div class=\"cuerpo\">","</div>");
        $content =~ s/<br *\/>//g;

        my $date = $year;

        print "<entry>\n";
        print "  <title>" . $title . "</title>\n";
        print "  <date>" . $date . "</date>\n";
        print "  <content>" . $content . "</content>\n";
        print "  <link href='" . $url . "'/>\n";
        print "  <media:group>\n";
        print "     <media:thumbnail url='" . $thumbnail . "'/>\n";
        print "     <media:player url='" . $url . "'/>\n";
        print "  </media:group>\n";
        print "</entry>\n";

        # add to array
        my $moviename = $title;
        if ($year ne "") {
            $moviename .= " ($year)";
        }
        $movies[$count++] = $movienum . ":" . $moviename;
     }
   }

   print "</feed>";
}

# get video page link and return this.
sub getMediaURL {

   my ($filename)=@_; # grab parameters
   my $url = $filename;

   # get the search results  page
   my $ua = LWP::UserAgent->new;
   $ua->timeout(10);
   $ua->env_proxy;

   my $response = $ua->get($url);
   my $data = parseBetween($response->content,"videoFile: '","'");

   my $enclosureURL_cont = "http://trailers.alpacine.com/$data";
   print $url . " " . $enclosureURL_cont;
}


#
# Main Program
#

# parse command line arguments 
getopts('hrdviSM');

# print out info 
if (defined $opt_v) { version(); exit 1; }
if (defined $opt_i) { info(); exit 1; }

# print out usage if needed
if (defined $opt_h || $#ARGV<0) { help(); }

# search
elsif (defined $opt_S) {
   # take query from cmdline arg
   $options = shift || die "Usage : $0 -S <query>\n";
   $query = shift;
   if (!$query) {
      $query = $options;
      $options = "";
   }
   search($query);
}
# get media url
elsif (defined $opt_M) {
   # take video page link
   $options = shift || die "Usage : $0 -M <video_page>\n";
   $page = shift;
   if (!$page) {
      $page = $options;
      $options = "";
   }

   getMediaURL($page);
}
