/* ============================================================
 * File  : searchchannel.cpp
 * Author: Alvaro Gonzalez Sola <cojonuo@gmail.com>
 * Date  : 2008-07-30
 * Description :
 *
 * Interface to SearchScreen, provide function to search, view and manage
 * videos from video sites.
 *
 * Copyright 2008 by Alvaro Gonzalez Sola
 *
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published bythe Free Software Foundation;
 * either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * ============================================================ */

#ifndef SEARCHSCREEN_CPP
#define SEARCHSCREEN_CPP

// C Headers
#include <unistd.h>
#include <math.h>

// QT headers
#include <qapplication.h>
#include <q3network.h>
#include <qdir.h>

//Added by qt3to4:
#include <QUrl>

// MythTV headers
#include <mythtv/libmythui/mythmainwindow.h>
#include <mythtv/util.h>
#include <mythtv/httpcomms.h>
#include <mythtv/mythcontext.h>
#include <mythtv/mythdbcon.h>
#include <mythtv/mythdirs.h>

using namespace std;

#include "searchscreen.h"

#include <qfile.h>
#include <qstringlist.h>
#include <qdatastream.h>

// 

//========================================================================================
//              Constructor, Desctructor and setup function
//========================================================================================

SearchScreen::SearchScreen(MythScreenStack *parent, const char *name)
    : MythTubeScreen (parent, name)
{

    // Initialize variables

    m_queryEdit = NULL;
    m_searchButton = NULL;
    m_videoList = NULL;
    m_siteList = NULL;
    m_infoText = NULL;
    m_timeText = NULL;
    m_dateText = NULL;
    m_ratingText = NULL;

    m_thumbnailImage = NULL;
    m_menuPopup = NULL;

    search = NULL;

// OLD CODE
/*
//     m_progressPopup = NULL;
//     httpGrabber = NULL;
//     cacheSize = 4;
//     cacheIndex = 0;
*/
}

// OLD CODE
/*
// QString SearchScreen::ConfDir(QString path = NULL){
//     QString fileprefix = GetConfDir();
// 
//     QDir dir(fileprefix);
//     if (!dir.exists())
//         dir.mkdir(fileprefix);
//     fileprefix += "/SearchScreen";
//     dir = QDir(fileprefix);
//     if (!dir.exists())
//         dir.mkdir(fileprefix);
// 
//     if(!path.isEmpty()){
//         QStringList dir_list = QStringList::split("/" ,path);
//         for(int i = 0; i < dir_list.count(); i++){
//             fileprefix += "/" + dir_list[i];
//             QDir dir_aux(fileprefix);
//             if (!dir_aux.exists())
//                 dir_aux.mkdir(fileprefix);
//         }
//     }
// 
//     return fileprefix;
// }
*/

SearchScreen::~SearchScreen() { }

bool SearchScreen::Create(){

    bool foundtheme = false;

    foundtheme = LoadWindowFromXML("tube-ui.xml", "tube", this);

    if (!foundtheme)
        return false;

    m_queryEdit = dynamic_cast<MythUITextEdit *> (GetChild("query"));
    m_searchButton = dynamic_cast<MythUIButton *> (GetChild("search"));
    m_videoList = dynamic_cast<MythUIButtonList *> (GetChild("videolist"));
    m_siteList = dynamic_cast<MythUIButtonList *> (GetChild("sitelist"));
    m_infoText = dynamic_cast<MythUIText *> (GetChild("infotext"));
    m_timeText = dynamic_cast<MythUIText *> (GetChild("timetext"));
    m_dateText = dynamic_cast<MythUIText *> (GetChild("datetext"));  
    m_ratingText = dynamic_cast<MythUIText *> (GetChild("ratingtext"));
    m_thumbnailImage = dynamic_cast<MythUIImage *> (GetChild("thumbnail"));

    if (!m_queryEdit || !m_searchButton || !m_videoList ||
        !m_siteList || !m_infoText || !m_timeText || !m_dateText ||
        !m_ratingText || !m_thumbnailImage)
    {
        VERBOSE(VB_IMPORTANT, "Theme is missing critical theme elements.");
        return false;
    }

    if (!BuildFocusList())
        VERBOSE(VB_IMPORTANT, "Failed to build a focuslist. Something is wrong");

    m_videoList->SetActive(false);

    getSitesScript();

    connect(m_videoList, SIGNAL(itemSelected( MythUIButtonListItem*)),
            this, SLOT( updateInfoView(MythUIButtonListItem*)));
    connect(m_searchButton, SIGNAL(buttonPressed()), this, SLOT(slotSearch()));
    connect(m_videoList, SIGNAL(itemClicked( MythUIButtonListItem*)),
            this, SLOT( selectVideo(MythUIButtonListItem*)));
    //genera una lista de elementos con foco para poder pasar por ellos

    SetFocusWidget(m_queryEdit);

    return true;
}

bool SearchScreen::keyPressEvent(QKeyEvent *event)
{
    if (GetFocusWidget()->keyPressEvent(event))
        return true;

    bool handled = false;
    QStringList actions;
    gContext->GetMainWindow()->TranslateKeyPress("Tube", event, actions);

    for (int i = 0; i < actions.size() && !handled; i++)
    {
        QString action = actions[i];
        handled = true;

        if (action == "MENU"){
                ShowMenu();
        }else{
            handled = false;
        }
    }

    if (!handled && MythTubeScreen::keyPressEvent(event))
        handled = true;

    return handled;
}



//========================================================================================
//          Slots
//========================================================================================

void SearchScreen::slotSearch()
{
    QString query = m_queryEdit->GetText();
    QString script = m_siteList->GetItemCurrent()->text();

    search = new Search(query,script);

    search->executeSearch();
    connect(search, SIGNAL(finishedSearch(Search*)),
                this, SLOT(showResult(Search*)));
    connect(search, SIGNAL(finishedMoreInfo(TubeVideo *)),
                this, SLOT(slotViewVideo(TubeVideo *)));

    showInfo("searching...");
}

void SearchScreen::showResult(Search* search){

// OLD CODE
/*
//     if (m_infoPopup)
//             {
//         m_infoPopup->Close();
//         m_infoPopup = NULL;
//     }
*/

    clearInfo();

    search->process();

    m_videoList->Reset();

    if(search->videoList().count() > 0){

        for (TubeVideo* video = search->videoList().first(); video;
                video = search->videoList().next()) {

            MythUIButtonListItem* item =
                new MythUIButtonListItem(m_videoList, video->title());
            item->setData(video);
        }

    }else{
        showInfo("No result");
    }
}

void SearchScreen::updateInfoView(MythUIButtonListItem* selected)
{
    if (!selected)
        return;

    TubeVideo *video = NULL;

    if (m_thumbnailImage)
    {
        m_thumbnailImage->Hide();
    }

     if (GetFocusWidget() == m_videoList)
     {
         video = (TubeVideo*) selected->getData();

        if(video){
             m_infoText -> SetText(video->description());
             m_timeText -> SetText("duration: " + video->time() + " sec");
             m_dateText -> SetText("date: " + video->date());
             m_ratingText -> SetText("view count: " + video->rating());
            if (!video->thumbnail().isEmpty())
            {

                QString fileprefix = configDir("/Tubecache");

                QString url = video->thumbnail();
                QString sFilename = QString("%1/%2")
                        .arg(fileprefix).arg(qChecksum(url,url.length()));

                bool exists = QFile::exists(sFilename);
                if (!exists)
                    HttpComms::getHttpFile(sFilename, url, 20000);

                if (m_thumbnailImage)
                {
                    m_thumbnailImage->SetFilename(sFilename);
                    m_thumbnailImage->Load();

                    if (!m_thumbnailImage->IsVisible())
                        m_thumbnailImage->Show();
                }
            }
        }
     }
}




//========================================================================================
//          Private functions (Download and play media)
//========================================================================================

/* void SearchScreen::createProgress(QString title)
// {
//     if (m_progressPopup)
//         return;
// 
//     QString message = title;
// 
//     MythScreenStack *popupStack = GetMythMainWindow()->GetStack("popup stack");
// 
//     m_progressPopup = new MythUIProgressDialog(message, popupStack,
//                                                "mythtubeprogressdialog");
// 
//     if (m_progressPopup->Create())
//         popupStack->AddScreen(m_progressPopup, false);
// }
*/ 
/* bool SearchScreen::getHttpFile(QString sFilename, QString cmdURL)
// {
//     int redirectCount = 0;
//     int timeoutCount = 0;
//     QByteArray data(0);
//     bool res = false;
//     httpGrabber = NULL;
//     QString hostname = "";
// 
//     VERBOSE(VB_IMPORTANT, cmdURL);
// 
//     createProgress(QObject::tr("Downloading media..."));
//     while (1)
//     {
//         QUrl qurl(cmdURL);
//         if (hostname == "")
//             hostname = qurl.host();  // hold onto original host
// 
//         if (!qurl.hasHost())        // can occur on redirects to partial paths
//             qurl.setHost(hostname);
// 
//         if (httpGrabber != NULL)
//             delete httpGrabber;
// 
//         httpGrabber = new HttpComms;
//         abortHttp = false;
// 
//         httpGrabber->request(qurl, -1, true);
// 
//         while ((!httpGrabber->isDone()) && (!abortHttp))
//         {
//             int total = httpGrabber->getTotal();
//             m_progressPopup->SetTotal(total);
//             int progress = httpGrabber->getProgress();
//             m_progressPopup->SetProgress(progress);
//             qApp->processEvents();
//             usleep(100000);
//         }
// 
//         if (abortHttp)
//             break;
// 
//         // Check for redirection
//         if (!httpGrabber->getRedirectedURL().isEmpty())
//         {
//             if (redirectCount++ < 5)
//                 cmdURL = httpGrabber->getRedirectedURL();
// 
//             // Try again
//             timeoutCount = 0;
//             continue;
//         }
// 
//         data = httpGrabber->getRawData();
// 
//         if (data.size() > 0)
//         {
//             QFile file(sFilename);
//             if (file.open( QIODevice::WriteOnly ))
//             {
//                 QDataStream stream(& file);
//                 stream.writeRawBytes( (const char*) (data), data.size() );
//                 file.close();
//                 res = true;
//             }
//         }
//         break;
//     }
// 
//     if (m_progressPopup)
//     {
//         m_progressPopup->Close();
//         m_progressPopup = NULL;
//     }
// 
//     delete httpGrabber;
//     httpGrabber = NULL;
//     return res;
// 
// }*/

//comprueba si el video tiene la direccion o es necesario obtenerla
void SearchScreen::selectVideo(MythUIButtonListItem* selected){

    if (!selected)
        return;

    TubeVideo *video = (TubeVideo*) selected->getData();
    if(video)
    {

        if (!video->enclosure().isEmpty())
        {
            slotViewVideo(video);
        }
        else {
            VERBOSE(VB_GENERAL, QString("no hay video"));
            search->getMoreInfo(video);
        }

    }
}


// descarga y cachea un video
/*  void SearchScreen::downloadVideo(TubeVideo* video, QString path = NULL)
// {
// 
//     QString cmdURL(video->enclosure());
// 
//     QString fileprefix = configDir();
// 
//     QString sFilename(fileprefix + "/videotempfile");
// 
//     sFilename += QString::number(cacheIndex);
// 
//     VERBOSE(VB_GENERAL, QString("downloading on ")+sFilename);
// 
//     if (getHttpFile(sFilename, cmdURL))
//     {
//         cache.remove(cache.key(sFilename));
//         cache.insert(video->tubeURL(),sFilename);
//         cacheIndex++;
//         cacheIndex = cacheIndex % cacheSize;
// 
//         if(!path.isEmpty()){
//             QFile file(sFilename);
//             file.copy(path + "/" + video->title());
//         }
// 
//         qApp->unlock();
//         playVideo(sFilename);
//         qApp->lock();
//     }
// }
//
*/

//comprueba si el video esta en cache o es necesario descargarlo
// punto de entrada tras obtener la url del medio
void SearchScreen::slotViewVideo(TubeVideo* video){

    //si esta en cache lo uso de ahi de ahi
    if(onCache(video)){
        VERBOSE(VB_GENERAL, QString("video cacheado"));

        QString file = getCache(video);

        if(m_mode == SAVE){
            m_mode = DOWNLOAD;
            VERBOSE(VB_GENERAL, QString("guardando Video"));

            QString fileprefix = configDir("/saveVideos");

            QFile savefile(file);

            savefile.copy(fileprefix + "/" + video->title());

        }else{


            qApp->unlock();
            playVideo(file);
            qApp->lock();
        }

    }
    //si es un stream lo reproduzco
    else if(m_mode == STREAM){
        m_mode = DOWNLOAD;
        VERBOSE(VB_GENERAL, QString("reproduciendo Stream"));

        qApp->unlock();
        playVideo("\""+video->enclosure()+"\"");
        qApp->lock();

    }else if(m_mode == SAVE){
        m_mode = DOWNLOAD;
        VERBOSE(VB_GENERAL, QString("guardando Video"));

        QString fileprefix = configDir("/saveVideos");

        downloadVideo(video, fileprefix);
    }else{
        downloadVideo(video);
    }

}

//reproduce un video apartir de una direccion
/* void SearchScreen::playVideo(const QString &filename)
// {
//     QString command_string = gContext->GetSetting("VideoDefaultPlayer");
// 
//     gContext->sendPlaybackStart();
// 
//     if ((command_string.find("Internal", 0, false) > -1) ||
//         (command_string.length() < 1))
//     {
//         command_string = "Internal";
//         gContext->GetMainWindow()->HandleMedia(command_string, filename);
//     }
//     else
//     {
//         if (command_string.contains("%s"))
//             command_string = command_string.replace(QRegExp("%s"), filename);
// 
//         myth_system(command_string);
//     }
// 
//     gContext->sendPlaybackEnd();
// } 
*/

void SearchScreen::getSitesScript(){
    QString path = GetShareDir();
    path += "mythtube/scripts/";

    QDir* dir = new  QDir(path); 
    QStringList site_list = dir->entryList(QDir::Files);

    for(int i=0; i<site_list.size(); i++){
        new MythUIButtonListItem(m_siteList, site_list[i]);
    }
}

void SearchScreen::ShowMenu(){
    QString label = tr("Options");

    MythScreenStack *popupStack =
                            GetMythMainWindow()->GetStack("popup stack");

    m_menuPopup = new MythDialogBox(label, popupStack, "mythtubemenupopup");

    if (m_menuPopup->Create())
        popupStack->AddScreen(m_menuPopup);

    m_menuPopup->SetReturnEvent(this, "options");

    m_menuPopup->AddButton(tr("View Stream"));
    m_menuPopup->AddButton(tr("Save"));
    m_menuPopup->AddButton(tr("Cancel"));
}

// OLD CODE
/*
// void SearchScreen::ShowInfo(QString message){
//     QString label = tr(message);
// 
//     MythScreenStack *popupStack =
//                             GetMythMainWindow()->GetStack("popup stack");
// 
//     m_infoPopup = new MythDialogBox(label, popupStack, "mythtubeinfopopup");
// 
//     if (m_infoPopup->Create())
//         popupStack->AddScreen(m_infoPopup);
// } 
*/

void SearchScreen::customEvent(QEvent *event)
{

    if (event->type() == kMythDialogBoxCompletionEventType)
    {
        DialogCompletionEvent *dce =
                                dynamic_cast<DialogCompletionEvent*>(event);

        QString resultid= dce->GetId();
        int buttonnum  = dce->GetResult();

        if (resultid == "options")
        {
            if (buttonnum == 0)
            {
                StreamVideo();
            }
            else if (buttonnum == 1)
            {
                SaveVideo();
            }
        }

        m_menuPopup = NULL;
    }

}

//lanza una reproduccion activando el stream
void SearchScreen::StreamVideo(){

    m_mode = STREAM;
    selectVideo(m_videoList->GetItemCurrent());

}

//salva un video con su titulo y en el directorio por defecto
void SearchScreen::SaveVideo(){

    m_mode = SAVE;
    selectVideo(m_videoList->GetItemCurrent());

}


#endif
